; IDL DISLIN wrapper shortcut procedure to draw an arrow
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_arrow, pos_x, pos_y, vec_x, vec_y, log=log, line=line, width=width, color=color, ratio=ratio, opening=opening, bare=bare, reverse=reverse
	if (size (log, /type) eq 0) then log = ''
	if (size (log, /type) ne 7) then if (keyword_set (log)) then log = 'xy' else log = ''
	if (size (line, /type) eq 0) then line = 'solid'
	if (not keyword_set (color)) then color = 'black'
	if ((n_elements (color) ne 3) and (size (color, /type) ne 7)) then color = 'blue'
	if (size (ratio, /type) eq 0) then ratio = 0.25
	if (size (opening, /type) eq 0) then opening = 30

	if (n_params () eq 2) then begin
		vec_x = pos_x
		vec_y = pos_y
		num_x = 0
		num_y = 0
	end else begin
		num_x = n_elements (pos_x)
		num_y = n_elements (pos_y)
	end
	dims = size (vec_x, /dimensions)
	num_dims = n_elements (dims)
	num = n_elements (vec_x)
	if (num gt 1) then begin
		if (num_x eq 0) then begin
			dislin, 'getgrf', xs, xe, xf, xd, 'X'
			pos_x = [ xs, xe ]
			num_x = 2
		end
		if (num_y eq 0) then begin
			dislin, 'getgrf', ys, ye, yf, yd, 'Y'
			pos_y = [ ys, ye ]
			num_y = 2
		end
		if ((num gt 4) and (num_dims eq 2) and (num_x eq 2) and (num_y eq 2)) then begin
			num_x = dims[0]
			num_y = dims[1]
			pos_x = (dindgen (num_x) + 0.5) / num_x * (pos_x[1] - pos_x[0]) + pos_x[0]
			pos_y = (dindgen (num_y) + 0.5) / num_y * (pos_y[1] - pos_y[0]) + pos_y[0]
		end
		if ((num eq num_x*num_y) and (size (pos_x, /n_dimensions) eq 1) and (size (pos_y, /n_dimensions) eq 1)) then begin
			coord_x = dblarr (num_x, num_y)
			coord_y = dblarr (num_x, num_y)
			for py = 0, num_y-1 do coord_x[*,py] = pos_x
			for px = 0, num_x-1 do coord_y[px,*] = pos_y
			dislin_arrow, coord_x, coord_y, vec_x, vec_y, log=log, line=line, width=width, color=color, ratio=ratio, opening=opening, bare=bare, reverse=reverse
			return
		end
		if ((n_elements (vec_y) ne num) or (num_x ne num) or (num_y ne num)) then begin
			print, 'WARNING: nothing plotted because vector arrays differ in size.'
			return
		end
		for pos = 0, num-1 do begin
			dislin_arrow, pos_x[pos], pos_y[pos], vec_x[pos], vec_y[pos], log=log, line=line, width=width, color=color, ratio=ratio, opening=opening, bare=bare, reverse=reverse
		end
		return
	end

	x = [ pos_x[0], pos_x[0]+vec_x[0] ]
	y = [ pos_y[0], pos_y[0]+vec_y[0] ]
	if (keyword_set (reverse)) then begin
		x = reverse (x)
		y = reverse (y)
	end
	if (strpos (log, 'x') ge 0) then x = 10^x
	if (strpos (log, 'y') ge 0) then y = 10^y

	if (not keyword_set (bare)) then dislin_line, x, y, line=line, width=width, color=color

	plot_x = float (x)
	plot_y = float (y)
	dislin, 'trfrel', plot_x, plot_y, 2
	plot_dx = double (plot_x[1] - plot_x[0])
	plot_dy = double (plot_y[1] - plot_y[0])
	if (plot_dx eq 0) then begin
		if (plot_dy ge 0) then angle = !DPI/2 else angle = -!DPI/2
	end else begin
		angle = atan (plot_dy / plot_dx)
		if (plot_dx lt 0) then angle += !DPI
	end

	length = ratio * sqrt (plot_dx^2 + plot_dy^2)
	sin_opening = sin (opening * !DPI/180)
	cos_opening = cos (opening * !DPI/180)

	start = [ plot_x[0], plot_y[0] ]
	tip = [ plot_x[1], plot_y[1] ]
	end_upper = tip + [0,1] * sin_opening * length - [1,0] * cos_opening * length
	end_lower = tip - [0,1] * sin_opening * length - [1,0] * cos_opening * length
	end_upper = dislin_rotate (end_upper - tip, angle) + tip
	end_lower = dislin_rotate (end_lower - tip, angle) + tip

	if (size (width, /type) ne 0) then dislin, 'linwid', round (width)
	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end
	dislin_color, color
	dislin, 'line', round (tip[0]), round (tip[1]), round (end_upper[0]), round (end_upper[1])
	dislin, 'line', round (tip[0]), round (tip[1]), round (end_lower[0]), round (end_lower[1])
end

