; IDL DISLIN wrapper shortcut procedure to create a new curve
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_curve, x, y, z, good=good, stride=stride, color=color, pattern=pattern, symbol=symbol, size=size, line=line, mode=mode, width=width, xs=xs, xe=xe, ys=ys, ye=ye, zs=zs, ze=ze, legend=legend, title=title, histogram=histogram, interpolation=interpolation, bar_width=bar_width, signed=signed, log=log, pos_color=pos_color, neg_color=neg_color, pos_symbol=pos_symbol, neg_symbol=neg_symbol, pos_size=pos_size, neg_size=neg_size, pos_line=pos_line, neg_line=neg_line, pos_mode=pos_mode, neg_mode=neg_mode, pos_width=pos_width, neg_width=neg_width, pos_title=pos_title, neg_title=neg_title

	common dislin_skip_config, dislin_skip_page

	if (keyword_set (dislin_skip_page)) then return

	NaN = !Values.D_NaN
	Inf = !Values.D_Infinity

	if (keyword_set (signed)) then begin
		if ((size (pos_symbol, /type) eq 0) and (size (symbol, /type) ne 0)) then pos_symbol = symbol
		if ((size (neg_symbol, /type) eq 0) and (size (symbol, /type) ne 0)) then neg_symbol = symbol
		if ((size (pos_size, /type) eq 0) and (size (size, /type) ne 0)) then pos_size = size
		if ((size (neg_size, /type) eq 0) and (size (size, /type) ne 0)) then neg_size = size
		if ((size (pos_line, /type) eq 0) and (size (line, /type) ne 0)) then pos_line = line
		if ((size (neg_line, /type) eq 0) and (size (line, /type) ne 0)) then neg_line = line
		if ((size (pos_mode, /type) eq 0) and (size (mode, /type) ne 0)) then pos_mode = mode
		if ((size (neg_mode, /type) eq 0) and (size (mode, /type) ne 0)) then neg_mode = mode
		if ((size (pos_width, /type) eq 0) and (size (width, /type) ne 0)) then pos_width = width
		if ((size (neg_width, /type) eq 0) and (size (width, /type) ne 0)) then neg_width = width

		positive = where (y ge 0.0)
		negative = where (y le 0.0)
		y_positive = y
		if (negative[0] ne -1) then y_positive[negative] = NaN
		if (keyword_set (log)) then y_negative = -y else y_negative = y
		if (positive[0] ne -1) then y_negative[positive] = NaN

		x_positive = x
		x_negative = x
		last = n_elements (x) - 1
		for pos = 0, last do begin
			if ((y_positive[pos] gt 0.0) and not finite (y_positive[(pos+1)<last])) then begin
				y_positive[pos+1] = 0.0
				x_positive[pos+1] = x[pos]
				if ((pos+2 le last) and finite (y_positive[(pos+2)<last])) then x_positive[pos+1] = 0.5 * (x[pos] + x[pos+2])
			end
			if ((y_positive[pos] gt 0.0) and not finite (y_positive[(pos-1)>0])) then begin
				y_positive[pos-1] = 0.0
				x_positive[pos-1] = x[pos]
				if ((pos-2 ge 0) and finite (y_positive[(pos-2)>0])) then x_positive[pos-1] = 0.5 * (x[pos] + x[pos-2])
			end
			if ((abs (y_negative[pos]) gt 0.0) and not finite (y_negative[(pos+1)<last])) then begin
				y_negative[pos+1] = 0.0
				x_negative[pos+1] = x[pos]
				if ((pos+2 le last) and finite (y_negative[(pos+2)<last])) then x_negative[pos+1] = 0.5 * (x[pos] + x[pos+2])
			end
			if ((abs (y_negative[pos] gt 0.0)) and not finite (y_negative[(pos-1)>0])) then begin
				y_negative[pos-1] = 0.0
				x_negative[pos-1] = x[pos]
				if ((pos-2 ge 0) and finite (y_negative[(pos-2)>0])) then x_negative[pos-1] = 0.5 * (x[pos] + x[pos-2])
			end
		end

		if (keyword_set (log)) then begin
			if (n_elements (ys) eq 0) then log_ys = min (alog10 (abs (y))) else log_ys = ys
			log_ys = 10^(2*(log_ys - abs (log_ys))) > 1e-42
			indices_zero = where (y_positive eq 0.0, num_zero)
			if (num_zero ge 1) then y_positive[indices_zero] = log_ys
			indices_zero = where (y_negative eq 0.0, num_zero)
			if (num_zero ge 1) then y_negative[indices_zero] = log_ys
		end else begin
			; Extend positive and negative side to an interpolated zero at each sign reversal
		end

		dislin_curve, x_negative, y_negative, color=neg_color, stride=stride, symbol=neg_symbol, size=neg_size, line=neg_line, mode=neg_mode, width=neg_width, xs=xs, xe=xe, ys=ys, ye=ye, legend=legend, title=neg_title, histogram=histogram
		dislin_curve, x_positive, y_positive, color=pos_color, stride=stride, symbol=pos_symbol, size=pos_size, line=pos_line, mode=pos_mode, width=pos_width, xs=xs, xe=xe, ys=ys, ye=ye, legend=legend, title=pos_title, histogram=histogram
		return
	end

	if (keyword_set (histogram)) then begin
		if (size (mode, /type) eq 0) then mode = 0
		if (size (line, /type) eq 0) then line = 1
	end

	elements = n_elements (x)
	if (elements eq 0) then begin
		print, 'WARNING: nothing plotted because X array is empty.'
		return
	end
	elements_z = n_elements (z)

	if (keyword_set (histogram)) then begin
		if ((n_elements (y) ne elements-1) and (n_elements (y) ne elements)) then begin
			print, 'WARNING: nothing plotted because X and Y do not match a histogram: ', elements, n_elements (y)
			return
		end
		hist_x = dblarr (elements * 2)
		hist_y = dblarr (elements * 2)
		for pos = 0, elements-1 do hist_x[2*pos:2*pos+1] = x[pos]
		for pos = 0, elements-2 do hist_y[2*pos+1:2*pos+2] = y[pos]
		x = hist_x
		y = hist_y
		elements *= 2
	end

	if (n_elements (y) ne elements) then begin
		print, 'WARNING: nothing plotted because X and Y arrays differ in size: ', elements, n_elements (y)
		return
	end
	if ((elements_z ne 0) and (elements_z ne elements)) then begin
		print, 'WARNING: nothing plotted because Z array differs in size.'
		return
	end

	if (size (good, /type) eq 0) then begin
		good = finite (x) and finite (y)
		if (elements_z ne 0) then good = good and finite (z)
	end else if (n_elements (good) ne elements) then begin
		print, 'WARNING: nothing plotted because X and GOOD arrays differ in size: ', elements, n_elements (good)
		return
	end

	if (size (title, /type) ne 7) then title = ''
	if (title ne '') then begin
		if (size (legend, /type) ne 7) then legend = title else legend = [ legend, title ]
	end

	normal = where (good, num_normal)
	if (num_normal eq 0) then begin
		print, 'WARNING: nothing plotted because all data is denormal.'
		return
	end
	if (num_normal ne elements) then begin
		pos = normal[0]
		first = pos
		last = elements - 1
		while (first le last) do begin
			while (good[pos] and (pos lt last)) do pos++
			if (good[pos]) then pos++
			tmp_x = x[first:(pos-1)>first]
			tmp_y = y[first:(pos-1)>first]
			if ((pos - 1) gt first) then begin
				if (elements_z eq 0) then begin
					dislin_curve, tmp_x, tmp_y, stride=stride, color=color, symbol=symbol, size=size, line=line, mode=mode, width=width, xs=xs, xe=xe, ys=ys, ye=ye
				end else begin
					tmp_z = z[first:(pos-1)>first]
					dislin_curve, tmp_x, tmp_y, tmp_z, stride=stride, color=color, symbol=symbol, size=size, line=line, mode=mode, width=width, xs=xs, xe=xe, ys=ys, ye=ye, zs=zs, ze=ze
				end
			end
			if (pos gt last) then break
			while (not good[pos] and (pos lt last)) do pos++
			if (not good[pos]) then pos++
			first = pos > (first + 1)
		end
		return
	end

	indices = lindgen (elements)
	if (size (xs, /type) ne 0) then begin
		restrict = where (x lt xs)
		if (restrict[0] ne -1) then indices[restrict] = -1
	end
	if (size (xe, /type) ne 0) then begin
		restrict = where (x gt xe)
		if (restrict[0] ne -1) then indices[restrict] = -1
	end
	if (size (ys, /type) ne 0) then begin
		restrict = where (y lt ys)
		if (restrict[0] ne -1) then indices[restrict] = -1
	end
	if (size (ye, /type) ne 0) then begin
		restrict = where (y gt ye)
		if (restrict[0] ne -1) then indices[restrict] = -1
	end
	if (elements_z ne 0) then begin
		if (size (zs, /type) ne 0) then begin
			restrict = where (z lt zs)
			if (restrict[0] ne -1) then indices[restrict] = -1
		end
		if (size (ze, /type) ne 0) then begin
			restrict = where (z gt ze)
			if (restrict[0] ne -1) then indices[restrict] = -1
		end
	end
	restrict = 0
	indices = where (indices ge 0, num_good)
	if (num_good le 0) then begin
		print, 'WARNING: nothing plotted because no data is within bounds.'
		return
	end

	if (keyword_set (stride)) then begin
		if (stride ge 1) then indices = indices[0:*:stride+1]
		num_good = n_elements (indices)
	end

	if (size (color, /type) ne 0) then dislin_color, color
	if (size (interpolation, /type) eq 7) then begin
		dislin, 'polcrv', interpolation
		if (keyword_set (bar_width)) then dislin, 'barwth', double (bar_width)
	end

	if (elements_z eq 0) then begin
		if (n_elements (width) eq 1) then dislin, 'linwid', round (width)
		if (n_elements (mode) eq 1) then dislin, 'incmrk', round (mode)
		if (n_elements (symbol) eq 1) then dislin, 'marker', round (symbol)
		if (n_elements (size) eq 1) then dislin, 'hsymbl', round (size)
		if (n_elements (line) eq 1) then begin
			if (size (line, /type) ne 7) then line = 'solid'
			dislin, line
		end else if (n_elements (line) ge 2) then begin
			dislin, 'myline', round (line), n_elements (line)
		end
		if (size (pattern, /type) eq 0) then begin
			dislin, 'curve', double (x[indices]), double (y[indices]), num_good
		end else begin
			dislin, 'shdpat', round (pattern)
			dislin, 'shdcrv', double (x[indices]), double (y[indices]), num_good, double (minmax (x[indices])), double ([0,0]), 2
		end
	end else begin
		if (n_elements (size) eq 1) then dislin, 'setres', round (size), round (size)
		dislin, 'curve3', double (x[indices]), double (y[indices]), double (z[indices]), num_good
	end
end

