; IDL DISLIN wrapper shortcut procedure to create a new density plot
; Version 1.6, 31.03.2017, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_density, x, y, z, pos_x=pos_x, pos_y=pos_y, spacing=spacing, line=line, width=width, pattern=pattern, outline=outline, inverse=inverse, color=color, draw_background=draw_background, zs=zs, ze=ze

	if (n_params () eq 1) then begin
		if ((size (pos_x, /type) eq 0) or (size (pos_x, /type) eq 0)) then begin
			print, 'WARNING: nothing plotted because X or Y coordinates are not given.'
			return
		end
		z = temporary (x)
		x = pos_x
		y = pos_y
	end

	dims_z = size (z, /n_dimensions)
	if ((dims_z ne 2) and (dims_z ne 3)) then begin
		print, 'WARNING: nothing plotted because Z is not a density matrix.'
		return
	end

	size_z = size (z, /dimensions)
	tmp = double (minmax (z))
	min_z = tmp[0]
	max_z = tmp[1]
	if (dims_z eq 3) then begin
		if ((size_z[2] ne 3) or (min_z lt 0.0) or (max_z gt 1.0)) then begin
			print, 'WARNING: nothing plotted because Z contains invalid color information.'
			return
		end
	end
	elements_x = n_elements (x) - 1
	elements_y = n_elements (y) - 1
	if ((elements_x ne size_z[0]) or (elements_y ne size_z[1])) then begin
		print, 'WARNING: nothing plotted because Z differs in size from [X,Y]'
		print, elements_x, elements_y
		print, size_z
		return
	end

	if (size (spacing, /type) eq 0) then spacing = 0
	if (size (spacing, /n_elements) eq 1) then spacing = replicate (spacing, 2)
	spacing_lower = spacing/2
	spacing_upper = spacing - spacing_lower

	if (not keyword_set (color)) then color = 0
	if (not keyword_set (inverse)) then inverse = 0
	if (not keyword_set (draw_background)) then draw_background = 0

	if (n_elements (width) eq 1) then dislin, 'linwid', round (width)
	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end
	if (size (pattern, /type) eq 0) then pattern = 16
	dislin, 'shdpat', pattern
	if (not keyword_set (outline)) then dislin, 'noarln'

	if (n_elements (pos_x) ne (elements_x+1)) then begin
		pos_x = intarr (elements_x+1)
		for px = 0, elements_x do begin
			dislin, 'xposn', x[px], return=pos, type=4
			pos_x[px] = round (pos)
		end
	end

	if (n_elements (pos_y) ne (elements_y+1)) then begin
		pos_y = intarr (elements_y+1)
		for py = 0, elements_y do begin
			dislin, 'yposn', y[py], return=pos, type=4
			pos_y[py] = round (pos)
		end
	end

	if (dims_z eq 2) then begin
		if (size (zs, /type) eq 0) then zs = 0.0
		if (size (ze, /type) eq 0) then ze = max_z
		if (inverse) then shades = ((ze - z) > 0.0) / (ze - zs) else shades = ((z - zs) > 0.0) / (ze - zs)
		if (color) then begin
			if (draw_background) then begin
				max_color = 253
				min_color = 1
			end else begin
				max_color = 254
				min_color = 0
			end
			shades = max_color - ((round (shades * max_color) > 0) < max_color) + min_color
		end
		for py = 0, elements_y-1 do begin
			for px = 0, elements_x-1 do begin
				shade = shades[px,py]
				if (color) then begin
					if (not draw_background and (shade le 0)) then continue
					dislin, 'setclr', shade
				end else begin
					if (not draw_background and (shade ge (1.0 - 1d-10))) then continue
					dislin, 'setrgb', shade, shade, shade
				end
				dislin, 'rectan', pos_x[px]+spacing_lower[0], pos_y[py+1]-spacing_lower[1], (pos_x[px+1]-pos_x[px])-spacing_upper[0], (pos_y[py+1]-pos_y[py])+spacing_upper[1]
			end
		end
	end else begin
		for py = 0, elements_y-1 do begin
			for px = 0, elements_x-1 do begin
				dislin, 'setrgb', z[px,py,0], z[px,py,1], z[px,py,2]
				dislin, 'rectan', pos_x[px]+spacing_lower[0], pos_y[py+1]-spacing_lower[1], (pos_x[px+1]-pos_x[px])-spacing_upper[0], (pos_y[py+1]-pos_y[py])+spacing_upper[1]
			end
		end
	end
end

