; IDL DISLIN wrapper shortcut procedure to create an embedded background
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_embed, data, file=file, cache=cache, zs=zs, ze=ze, native=native, offset=offset, target=target, resolution=resolution, window=window, frame_width=frame_width, true_color=true_color

	common dislin_axis_config, default_type, default_label_type, default_label_axis, default_offset, default_target
	common dislin_skip_config, dislin_skip_page

	if (keyword_set (dislin_skip_page)) then return

	default_window = 31
	default_filename = 'DISLIN_background.png'

	if (size (file, /type) ne 7) then file = default_filename
	if (size (frame_width, /type) eq 0) then frame_width = 0

	if (size (offset, /type) eq 0) then offset = default_offset
	if (n_elements (offset) eq 1) then offset = [ offset, offset ]
	if (size (target, /type) eq 0) then target = default_target
	if (n_elements (target) eq 1) then target = [ target, target ]

	if (((size (data, /type) ne 0) and not keyword_set (cache)) or not file_test (file, /regular)) then begin
		if (size (data, /type) eq 0) then message, "No data given, but need to plot embedded image..."
		if (n_elements (native) ne 2) then native = (size (data, /dimensions))[0:1]
		if (size (resolution, /type) eq 0) then resolution = native
		if (n_elements (zs) eq 1) then data = data > zs
		if (n_elements (ze) eq 1) then data = data < ze
		if ((resolution[0] ne native[0]) or (resolution[1] ne native[1])) then begin & stop & data = congrid (data, resolution[0], resolution[1], 1, /center, /interpolate) & end
		if (not keyword_set (true_color)) then true_color = 0

		if (n_elements (window) eq 1) then begin
			wset, window
			image = tvrd (true=true_color)
			WRITE_PNG, file, image
		end else begin
			num = 256
			red = bytarr (num)
			green = bytarr (num)
			blue = bytarr (num)
			for index = 0, num-1 do begin
				ic = round (index)
				dislin, 'getind', ic, fr, fg, fb
				red[index] = byte (fr * (num-1))
				green[index] = byte (fg * (num-1))
				blue[index] = byte (fb * (num-1))
			end
			dislin, 'getgrf', fzs, fze, fzf, fzd, "Z"
			dislin, 'getran', izs, ize
			dislin, 'getscl', ixlog, iylog, izlog
			if (izlog eq 1) then begin
				channel = bytscl (alog10 (data), min=fzs, max=fze, top=ize-izs) + byte (izs)
			end else begin
				channel = bytscl (data, min=fzs, max=fze, top=ize-izs) + byte (izs)
			end
			WRITE_PNG, file, channel, red, green, blue
		end
	end else begin
		if (query_image (file, image_info) ne 1) then begin
			print, 'WARNING: nothing plotted because the file contains no valid image'
			return
		end
		if (size (native, /type) eq 0) then native = image_info.dimensions[0:1]
		if (size (resolution, /type) eq 0) then resolution = native
	end

	if (n_elements (resolution) eq 2) then dislin, 'autres', round (resolution[0]), round (resolution[1])
	dislin, 'frame', round (frame_width)
	dislin, 'filopt', "ON", "SCALE"
	dislin, 'filbox', round (offset[0]), round (offset[1]-target[1]), round (target[0]), round (target[1])
	dislin, 'incfil', file

	if (not keyword_set (cache) and (file eq default_filename)) then file_delete, file
end

