; IDL DISLIN wrapper shortcut procedure to create a new histogram
; Version 1.6, 09.02.2017, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

function dislin_histogram, data, pos_x=pos_x, pos_y=pos_y, size=size, bins=bins, grid_data=grid_data, log=log, xs=xs, xe=xe, ys=ys, ye=ye, hs=hs, he=he

	dims = size (data, /n_dimensions)
	if (size (log, /type) eq 0) then log = 0
	if (n_elements (log) eq 1) then log = [ log, log ]

	x = data[*,0]
	if ((size (xs, /type) eq 0) or (size (xe, /type) eq 0)) then begin
		extrema = double (minmax (x))
		if (size (xs, /type) eq 0) then xs = extrema[0]
		if (size (xe, /type) eq 0) then xe = extrema[1]
	end
	good = where ((x ge xs) and (x le xe), elements_x)
	if (good[0] eq -1) then begin
		print, "WARNING: nothing computed, because X interval contains no data."
		return, !Values.D_NaN
	end
	x = x[good]
	lxs = xs
	lxe = xe
	if (log[0]) then begin
		x = alog10 (x)
		lxs = alog10 (lxs)
		lxe = alog10 (lxe)
	end

	if (dims eq 1) then begin
		y = !Values.D_NaN
		lys = !Values.D_NaN
		lye = !Values.D_NaN
		is_function = 0
	end else if (dims eq 2) then begin
		y = data[*,1]
		y = y[good]
		is_function = ((size (data, /dimensions))[1] eq 3)
		if (is_function) then begin
			z = data[*,2]
			z = z[good]
		end
		if ((size (ys, /type) eq 0) or (size (ye, /type) eq 0)) then begin
			extrema = double (minmax (y))
			if (size (ys, /type) eq 0) then ys = extrema[0]
			if (size (ye, /type) eq 0) then ye = extrema[1]
		end
		good = where ((y ge ys) and (y le ye), elements_y)
		if (good[0] eq -1) then begin
			print, "WARNING: nothing computed, because Y interval contains no data."
			return, !Values.D_NaN
		end
		x = x[good]
		y = y[good]
		if (is_function) then begin
			z = z[good]
			normalization = (1.0d0 / elements_x) * (1.0d0 / elements_y)
		end
		lys = ys
		lye = ye
		if (log[1]) then begin
			y = alog10 (y)
			lys = alog10 (lys)
			lye = alog10 (lye)
		end
	end else begin
		print, "WARNING: nothing computed, because a histogram can only take 1D or 2D data."
		return, !Values.D_NaN
	end

	delta = [ lxe-lxs, lye-lys ]

	if ((size (bins, /type) eq 0) and (size (size, /type) eq 0)) then begin
		if (dims eq 1) then begin
			bins = round (sqrt (elements_x)) > 1
		end else begin
			bins = (round (elements_x^0.25 * elements_y^0.25) > 1) < 250
		end
	end
	if (n_elements (bins) eq 1) then bins = [ bins, bins ]
	if (size (size, /type) eq 0) then size = delta / bins
	if (n_elements (size) eq 1) then size = [ size, size ]
	if (keyword_set (grid_data)) then begin
		lxs = lxs - 0.5 * delta[0]/bins[0]
		lxe = lxe + 0.5 * delta[0]/bins[0]
		lys = lys - 0.5 * delta[1]/bins[1]
		lye = lye + 0.5 * delta[1]/bins[1]
		delta = [ lxe-lxs, lye-lys ]
	end
	bins = round (delta / size)

	pos_x = lxs + dindgen (bins[0] + 1) / bins[0] * delta[0]

	if (dims eq 1) then begin
		pos_y = !Values.D_NaN
		density = lonarr (bins[0])
		for px = bins[0]-1, 0, -1 do begin
			good = where ((x ge pos_x[px]) and (x lt pos_x[px+1]), count, complement=complement)
			density[px] = count
			if (complement[0] ne -1) then x = x[complement]
		end
	end else begin
		pos_y = lys + dindgen (bins[1] + 1) / bins[1] * delta[1]
		bin_x = replicate (-1, elements_x)
		for px = bins[0]-1, 0, -1 do begin
			good = where ((x ge pos_x[px]) and (x lt pos_x[px+1]))
			if (good[0] ne -1) then bin_x[good] = px
		end
		bin_y = replicate (-1, elements_y)
		for py = bins[1]-1, 0, -1 do begin
			good = where ((y ge pos_y[py]) and (y lt pos_y[py+1]))
			if (good[0] ne -1) then bin_y[good] = py
		end
		if (is_function) then begin
			density = dblarr (bins[0], bins[1])
		end else begin
			density = lonarr (bins[0], bins[1])
		end
		for py = bins[1]-1, 0, -1 do begin
			for px = bins[0]-1, 0, -1 do begin
				good = where ((px eq bin_x) and (py eq bin_y), count, complement=complement)
				if (is_function) then begin
					if (count ge 1) then density[px,py] = total (z[good]) * normalization
				end else begin
					density[px,py] = count
				end
				if (complement[0] ne -1) then begin
					bin_x = bin_x[complement]
					bin_y = bin_y[complement]
				end
			end
		end
	end

	extrema = minmax (density)
	hs = extrema[0]
	he = extrema[1]

	if (log[0]) then pos_x = 10.0^pos_x
	if ((dims ge 2) and log[1]) then pos_y = 10.0^pos_y

	return, density
end

