; IDL DISLIN wrapper shortcut procedure to create a plot label
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_label, label, x, y, shift=shift, delta=delta, color=color, text_size=text_size, center=center, vertical=vertical, horizontal=horizontal, reset=reset, angle=angle, width=width, plot_coords=plot_coords, plot_origin=plot_origin

	common dislin_axis_config, default_type, default_label_type, default_label_axis, default_offset, default_target

	if (n_elements (width) eq 1) then dislin, 'linwid', round (width)

	num = n_elements (label)
	if (num lt 1) then begin
		print, 'WARNING: nothing plotted because label is empty.'
		return
	end

	if (size (x, /type) eq 0) then begin
		if (size (default_offset, /type) eq 0) then begin
			print, 'WARNING: nothing plotted because x position and default_offset are unknown.'
			return
		end
		x = replicate (default_offset[0], num)
		if (keyword_set (plot_origin)) then begin
			if (stregex (plot_origin, '(^|-)center(-|$)', /fold_case, /boolean)) then begin
				x += default_target[0] / 2
			end else if (stregex (plot_origin, '(^|-)right(-|$)', /fold_case, /boolean)) then begin
				x += default_target[0]
			end
		end
		plot_coords = 1
	end
	if (size (y, /type) eq 0) then begin
		if (size (default_offset, /type) eq 0) then begin
			print, 'WARNING: nothing plotted because y position and default_offset are unknown.'
			return
		end
		y = replicate (default_offset[1], num)
		if (keyword_set (plot_origin)) then begin
			if (stregex (plot_origin, '(^|-)middle(-|$)', /fold_case, /boolean)) then begin
				y -= default_target[1] / 2
			end else if (stregex (plot_origin, '(^|-)top(-|$)', /fold_case, /boolean)) then begin
				y -= default_target[1]
			end
		end
		plot_coords = 1
	end

	if ((size (color, /type) ne 0) and not keyword_set (color)) then color = 'black'
	if (size (color, /type) ne 0) then dislin_color, color
	if (n_elements (text_size) eq 1) then if (text_size eq 1) then dislin, 'reset', 'height' else dislin, 'height', round (text_size)
	if (n_elements (text_size) ge 2) then dislin, 'height', round (text_size[1])
	if (keyword_set (vertical) and (size (vertical, /type) ne 7)) then vertical = 'middle'
	if (keyword_set (horizontal) and (size (horizontal, /type) ne 7)) then horizontal = 'cent'
	if (keyword_set (center)) then begin
		if (size (vertical, /type) eq 7) then vertical = 'middle' else dislin, 'txtjus', 'middle'
		if (size (horizontal, /type) eq 7) then horizontal = 'cent' else dislin, 'txtjus', 'cent'
	end
	if (size (vertical, /type) eq 7) then dislin, 'txtjus', vertical
	if (size (horizontal, /type) eq 7) then dislin, 'txtjus', horizontal
	if (keyword_set (angle)) then dislin, 'angle', round (angle)
	if (size (shift, /type) eq 0) then shift = 0
	if (size (delta, /type) eq 0) then delta = 0
	if (n_elements (shift) eq 1) then shift_tmp = [ shift, shift ] else shift_tmp = shift
	if (n_elements (delta) eq 1) then delta_tmp = [ delta, delta ] else delta_tmp = delta

	for pos = 0, n_elements (label)-1 do begin
		x_pos = double ([ x[pos] + shift_tmp[0] ])
		y_pos = double ([ y[pos] + shift_tmp[1] ])
		if (not keyword_set (plot_coords)) then dislin, 'trfrel', x_pos, y_pos, 1
		x_pos = x_pos[0] + delta_tmp[0]
		y_pos = y_pos[0] + delta_tmp[1]
		text = label[pos]
		dislin, 'messag', text, round (x_pos), round (y_pos)
	end

	if (keyword_set (reset) or keyword_set (angle)) then dislin, 'angle', 0
	if (keyword_set (reset) or (size (vertical, /type) eq 7)) then dislin, 'txtjus', 'top'
	if (keyword_set (reset) or (size (horizontal, /type) eq 7)) then dislin, 'txtjus', 'left'
end

