; IDL DISLIN wrapper shortcut procedure for a least-absolute-deviation fit
; Version 1.6, 05.03.2017, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_ladfit, data_x, data_y, xs=xs, xe=xe, ys=ys, ye=ye, log=log, label=label, length=length, delta=delta, reference=reference, size=size, line=line, color=color, error_line=error_line, error_color=error_color, fit=fit, sigma=sigma, origin=origin
	if (size (log, /type) eq 0) then log = ''
	if (size (log, /type) ne 7) then if (keyword_set (log)) then log = 'xy' else log = ''
	if (size (line, /type) ne 7) then line = 'solid'
	if ((size (error_color, /type) eq 0) and (size (color, /type) ne 0)) then error_color = color
	if (not keyword_set (error_line)) then error_line = ''
	if (size (error_line, /type) ne 7) then error_line = 'dash'
	if (n_elements (origin) eq 1) then origin = [ origin, origin ]
	if (strpos (log, 'x') ge 0) then fit_x = alog10 (data_x) else fit_x = data_x
	if (strpos (log, 'y') ge 0) then fit_y = alog10 (data_y) else fit_y = data_y
	if (size (xs, /type) eq 0) then xs = min (fit_x)
	if (size (xe, /type) eq 0) then xe = max (fit_x)
	if (size (ys, /type) eq 0) then ys = min (fit_y)
	if (size (ye, /type) eq 0) then ye = max (fit_y)
	sorting = sort (fit_x)
	fit_x = fit_x[sorting]
	fit_y = fit_y[sorting]
	good = where (finite (fit_x) and finite (fit_y) and (fit_x ge xs) and (fit_x le xe) and (fit_y ge ys) and (fit_y le ye), num_good)
	if (num_good le 0) then begin
		print, "WARNING: nothing fitted because no data is within bounds."
		return
	end
	fit_x = fit_x[good]
	fit_y = fit_y[good]
	if (size (origin, /type) ne 0) then begin
		if (strpos (log, 'x') ge 0) then origin[0] = alog10 (origin[0])
		if (strpos (log, 'y') ge 0) then origin[1] = alog10 (origin[1])
		fit_x = [ origin[0], fit_x ]
		fit_y = [ origin[1], fit_y ]
	end
	fit = ladfit (fit_x, fit_y, absdev=sigma, /double)
	if (size (origin, /type) ne 0) then sigma *= (num_good + 1.0) / num_good
	num_fit_points = (ceil (xe - xs) + 1) > 2
	fit_x = xs + (xe - xs) * dindgen (num_fit_points) / (num_fit_points - 1)
	fit_y = fit[0] + fit[1] * fit_x
	if (strpos (log, 'y') ge 0) then fit_y = 10^fit_y
	if (strpos (log, 'x') ge 0) then fit_x = 10^fit_x
	if (error_line eq '') then begin
		dislin_line, fit_x, fit_y, line=line, color=color, slope=fit[1], label=label, length=length, delta=delta, reference=reference, size=size
	end else begin
		dislin_line, fit_x, fit_y, line=line, color=color
		if (strpos (log, 'x') ge 0) then fit_y_upper = fit_y*(1+sigma) else fit_y_upper = fit_y+sigma
		if (strpos (log, 'x') ge 0) then fit_y_lower = fit_y/(1+sigma) else fit_y_lower = fit_y-sigma
		dislin_line, fit_x, fit_y_upper, line=error_line, color=error_color, slope=fit[1], label=label, length=length, delta=delta, reference=reference, size=size
		dislin_line, fit_x, fit_y_lower, line=error_line, color=error_color
	end
end

