; IDL DISLIN wrapper shortcut procedure to create a new page
; Version 1.8, 05.10.2018, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_page, format, plot_file, page_width, page_height, scaling=scaling, hardware_font=hardware_font, text_font=text_font, text_size=text_size, tex_mode=tex_mode, tex_italics=tex_italics, screen_mode=screen_mode, page_mode=page_mode, page_format=page_format, page_frame=page_frame, page_fill=page_fill, color_table=color_table, file_mode=file_mode, color_mode=color_mode, error_file=error_file, error_device=error_device, error_mode=error_mode

	common dislin_page_config, default_text_font, default_text_size, default_tex_mode, default_tex_italics, default_hardware_font, default_output_format
	common dislin_skip_config, dislin_skip_page

	default_text_size = [ 36, 36, 30 ]
	default_tex_mode = 1
	default_tex_italics = "off"
	dislin_skip_page = 0

	if (size (scaling, /type) eq 0) then scaling = 1
	if (size (screen_mode, /type) eq 0) then screen_mode = 1
	if (size (screen_mode, /type) ne 7) then begin
		if (keyword_set (screen_mode)) then screen_mode = "revers" else screen_mode = "auto"
	end
	if (size (page_mode, /type) eq 0) then page_mode = 0
	if (size (page_mode, /type) ne 7) then begin
		if (keyword_set (page_mode)) then page_mode = "land" else page_mode = "none"
	end
	if (size (file_mode, /type) eq 0) then file_mode = (0 eq keyword_set (plot_file))
	if (size (file_mode, /type) ne 7) then begin
		if (keyword_set (file_mode)) then file_mode = "count" else file_mode = "delete"
	end
	if (size (error_device, /type) ne 7) then error_device = "cons"
	if ((size (page_format, /type) ne 7) and keyword_set (page_format)) then page_format = "da4l"
	if (size (color_mode, /type) eq 0) then color_mode = 1
	if (size (color_mode, /type) ne 7) then color_mode = 'RGB'

	num_params = n_params ()
	if (num_params eq 0) then format = ''
	if (format eq '') then format = 'cons'
	if (strcmp (format, 'x11', /fold_case)) then format = 'xwin'
	if (format eq 'skip') then dislin_skip_page = 1
	dislin, 'metafl', format
	default_output_format = format

	if ((num_params ge 2) and not max (strcmp (format, ['xwin','cons'], /fold_case)) and not keyword_set (dislin_skip_page)) then begin
		plot_filename = plot_file+'.'+format
		print, ' <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<'
		print, ' <<<<  Output: '+plot_filename
		dislin, 'setfil', plot_filename
		if (max (strcmp (format, ['png','tiff','bmp','image'], /fold_case))) then dislin, 'imgfmt', color_mode
	end
	dislin, 'filmod', file_mode
	dislin, 'scrmod', screen_mode
	if (keyword_set (page_format)) then begin
		dislin, 'setpag', page_format
	end else if (num_params ge 4) then begin
		dislin, 'page', round (page_width), round (page_height)
	end
	dislin, 'pagmod', page_mode
	dislin, 'errdev', error_device
	if (size (error_file, /type) eq 7) then dislin, 'errfil', error_file
	dislin, 'sclfac', double (scaling)
	dislin, 'disini'
	if (keyword_set (page_frame)) then dislin, 'pagera'
	if (keyword_set (page_fill)) then dislin, 'pagfll', round (page_fill)

	dislin_next, /initial, text_font=text_font, text_size=text_size, tex_mode=tex_mode, tex_italics=tex_italics, hardware_font=hardware_font

	if ((size (error_mode, /type) eq 7) and (size (error_mode, /n_dimensions) eq 2)) then begin
		num = n_elements (error_mode[0,*])
		for pos = 0, num - 1 do dislin, 'errmod', error_mode[0,pos], error_mode[1,pos]
	end
end

