; IDL DISLIN wrapper shortcut procedure to create a pie chart plot
; Version 1.5, 01.03.2016, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_pie, data, type=type, pattern=pattern, change=change, color=color, text_size=text_size, front_color=front_color, labels=labels, label_color=label_color, label_type=label_type, label_position=label_position, label_alignment=label_alignment, label_digits=label_digits, label_data_digits=label_data_digits, label_percent_digits=label_percent_digits, line=line, mode=mode, width=width

	elements = n_elements (data)
	if (elements lt 1) then begin
		print, 'WARNING: nothing plotted because the data array is empty.'
		return
	end

	if (size (labels, /type) eq 0) then begin
		buffer = ''
		num_labels = 0
	end else begin
		if (size (labels, /type) ne 7) then labels = strtrim (labels, 2)
		num_labels = n_elements (labels)
		if (num_labels mod elements ne 0) then begin
			print, 'WARNING: nothing plotted because LABELS has not an integer-multiple size: ', elements, num_labels
			return
		end
		dislin, 'legini', buffer, num_labels, max (strlen (labels))
		for i = 1, num_labels do begin
			index = i
			dislin, 'leglin', buffer, labels[i-1], index
		end
	end

	if ((size (type, /type) ne 7) and keyword_set (type)) then type = '3D'
	if (size (change, /type) eq 0) then change = 0
	if ((size (change, /type) ne 7) and keyword_set (change)) then change = 'pattern'
	if (size (color, /type) ne 0) then begin
		num_colors = n_elements (color)
		if (not keyword_set (change)) then change = 'none'
		if (size (front_color, /type) eq 0) then begin
			front_color = color
		end else begin
			if (size (type, /type) ne 7) then type = '3D'
		end
		if (num_colors ne n_elements (front_color)) then begin
			print, 'WARNING: nothing plotted because COLOR and FRONT_COLOR have to have the same size: ', num_colors, n_elements (front_color)
			return
		end
		dislin, 'pieclr', round (color), round (front_color), num_colors
	end
	if (n_elements (text_size) eq 1) then if (text_size eq 1) then dislin, 'reset', 'height' else dislin, 'height', round (text_size)
	if (n_elements (text_size) ge 2) then dislin, 'height', round (text_size[1])

	if (n_elements (width) eq 1) then dislin, 'linwid', round (width)
	if (n_elements (mode) eq 1) then dislin, 'incmrk', round (mode)
	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end

	if (size (change, /type) eq 7) then dislin, 'chnpie', change
	num_patterns = n_elements (pattern)
	if (num_patterns eq 1) then dislin, 'shdpat', round (pattern)
	if (size (pattern, /n_dimensions) eq 1) then begin
		for pos = 0, num_patterns-1 do dislin, 'patcyc', pos+1, round (pattern[pos])
	end

	if (size (label_color, /type) ne 0) then dislin, 'labclr', round (label_color), 'pie'
	if (size (label_type, /type) eq 7) then dislin, 'labels', label_type, 'pie'
	if (size (label_position, /type) eq 7) then dislin, 'labpos', label_position, 'pie'
	if (size (label_alignment, /type) eq 7) then dislin, 'labtyp', label_alignment, 'pie'
	if (size (label_digits, /type) ne 0) then dislin, 'labdig', round (label_digits), 'pie'
	if (size (label_data_digits, /type) ne 0) then dislin, 'labdig', round (label_data_digits), 'data'
	if (size (label_percent_digits, /type) ne 0) then dislin, 'labdig', round (label_percent_digits), 'percent'
	if (size (type, /type) eq 7) then dislin, 'pietyp', type
	dislin, 'piegrf', buffer, num_labels/elements, data, elements

	dislin_title
end

