; IDL DISLIN wrapper shortcut procedure to create a new plane
; Version 1.5, 17.03.2016, Philippe-A. Bourdin, Bourdin@MPS.mpg.de

pro dislin_plane, data, x, y, zs=zs, ze=ze, color=color, text_size=text_size, line=line, width=width, plane=plane, interpolation=interpolation, surface=surface, contours=contours, num_contours=num_contours, shade=shade, labels=labels, digits=digits, no_label=no_label, no_background=no_background

	if (size (data, /n_dimensions) ne 2) then begin
		print, 'WARNING: nothing plotted because DATA array not a 2D array.'
		return
	end
	data_size = size (data, /dimensions)
	num_x = data_size[0]
	num_y = data_size[1]
	if (num_x lt 2) then begin
		print, 'WARNING: nothing plotted because DATA array has less than 2 elements in X.'
		return
	end
	if (num_y lt 2) then begin
		print, 'WARNING: nothing plotted because DATA array has less than 2 elements in Y.'
		return
	end
	if (size (x, /type) ne 0) then begin
		if (size (y, /type) eq 0) then begin
			print, 'WARNING: nothing plotted because Y array is missing while X is given.'
			return
		end
		if ((n_elements (x) eq 2) and (num_x gt 2)) then x = x[0] + dindgen (num_x) / (num_x - 1.0) * (x[1] - x[0])
		if (n_elements (x) ne num_x) then begin
			print, 'WARNING: nothing plotted because X array differs in size: ', num_y, n_elements (y)
			return
		end
	end
	if (size (y, /type) ne 0) then begin
		if ((n_elements (y) eq 2) and (num_y gt 2)) then y = y[0] + dindgen (num_y) / (num_y - 1.0) * (y[1] - y[0])
		if (n_elements (y) ne num_y) then begin
			print, 'WARNING: nothing plotted because Y array differs in size: ', num_y, n_elements (y)
			return
		end
	end
	if (size (zs, /type) eq 0) then zs = min (data)
	if (size (ze, /type) eq 0) then ze = max (data)
	if (size (interpolation, /type) eq 0) then interpolation = 1
	if (n_elements (interpolation) eq 1) then interpolation = [ interpolation, interpolation ]

	num_colors = n_elements (color)
	array_color = size (color, /n_dimensions) ge 1
	scalar_color = (size (color, /type) eq 7) or ((size (color, /type) ne 4) and (size (color, /type) ne 5))
	if ((scalar_color and (num_colors eq 1)) or (not scalar_color and (num_colors eq 3))) then dislin_color, color
	if (n_elements (text_size) eq 1) then if (text_size eq 1) then dislin, 'reset', 'height' else dislin, 'height', round (text_size)
	if (n_elements (text_size) ge 2) then dislin, 'height', round (text_size[1])
	if (n_elements (width) eq 1) then dislin, 'linwid', round (width)
	if (n_elements (line) eq 1) then begin
		if (size (line, /type) ne 7) then line = 'solid'
		dislin, line
	end else if (n_elements (line) ge 2) then begin
		dislin, 'myline', round (line), n_elements (line)
	end

	if (keyword_set (plane) or (not keyword_set (surface) and not keyword_set (contours) and not keyword_set (num_contours))) then begin
		if (keyword_set (no_background)) then dislin, 'nobgd'
		dislin, 'crvmat', (double (data) < ze) > zs, num_x, num_y, round (interpolation[0]), round (interpolation[1])
		if (keyword_set (no_background)) then dislin, 'reset', "nobgd"
	end

	if (keyword_set (surface)) then begin
		if (keyword_set (x) and keyword_set (y)) then begin
			if (keyword_set (shade)) then begin
				if (size (shade, /type) eq 7) then dislin, 'shdmod', shade, "surface"
				dislin, 'surshd', x, num_x, y, num_y, (double (data) < ze) > zs
			end else begin
				dislin, 'surfce', x, num_x, y, num_y, (double (data) < ze) > zs
			end
		end else begin
			dislin, 'surmat', (double (data) < ze) > zs, num_x, num_y, round (interpolation[0]), round (interpolation[1])
		end
	end

	if (not keyword_set (num_contours)) then num_contours = n_elements (contours)
	if (num_contours ge 1) then begin
		if ((size (contours, /n_dimensions) eq 0) and (size (contours, /type) ne 4) and (size (contours, /type) ne 5)) then begin
			contours = zs + (dindgen (num_contours) * (ze - zs)) / (num_contours - 1.0d0)
			contours[num_contours-1] = ze
			if (ze lt zs) then begin
				tmp = zs
				zs = ze
				ze = tmp
			end
		end
		if (scalar_color and (num_colors eq 1)) then begin
			if ((size (color, /type) eq 7) and not array_color) then begin
				color = replicate (color, num_contours)
				num_colors = num_contours
			end else if (keyword_set (color)) then begin
				last_color = 255
				color = round (((1 + dindgen (num_contours)) * last_color) / num_contours)
				color[num_contours-1] = last_color
				num_colors = num_contours
			end
		end
		if (size (no_label, /type) eq 0) then no_label = 0
		if ((size (no_label, /n_dimension) eq 0) and (no_label[0] ge 2)) then no_label = [ no_label ]
		if (size (labels, /type) eq 0) then begin
			if ((size (no_label, /n_dimensions) ge 1) or not keyword_set (no_label)) then labels = 1 else labels = 0
		end
		num_labels = n_elements (labels)
		if (size (labels, /type) eq 7) then begin
			if (num_labels lt num_contours) then labels = [ labels, replicate ('', num_contours - num_labels) ]
			if (no_label[0] ge 1) then labels[no_label-1] = ''
		end else begin
			if (num_labels eq 1) then labels = replicate (labels, num_contours)
			if ((num_labels ge 2) and (num_labels lt num_contours)) then labels = [ labels, replicate (0, num_contours - num_labels) ]
			if (no_label[0] ge 1) then labels[no_label-1] = 0
		end
		num_labels = num_contours
		num_digits = n_elements (digits)
		if (num_digits eq 0) then digits = replicate (1, num_contours)
		if (num_digits eq 1) then digits = replicate (digits, num_contours)
		if ((num_digits ge 2) and (num_digits lt num_contours)) then digits = [ digits, replicate (1, num_contours - num_digits) ]
		num_digits = num_contours
		if (keyword_set (shade)) then begin
			if (size (shade, /type) eq 7) then dislin, 'shdmod', shade, "contur"
			dislin, 'conshd', x, num_x, y, num_y, double (data), double (contours), num_contours
			return
		end
		for pos = 0, num_contours-1 do begin
			if (scalar_color and (pos le num_colors-1)) then dislin_color, color[pos]
			if (not scalar_color and (num_contours*3 eq num_colors)) then dislin_color, [ color[0,pos], color[1,pos], color[2,pos] ]
			if (size (labels, /type) eq 7) then begin
				dislin, 'conlab', labels[pos]
				dislin, 'labels', "conlab", "contur"
			end else begin
				dislin, 'labdig', digits[pos], "contur"
				if (keyword_set (labels[pos])) then dislin, 'labels', "float", "contur" else dislin, 'labels', "none", "contur"
			end
			if (keyword_set (x) and keyword_set (y)) then begin
				dislin, 'contur', x, num_x, y, num_y, double (data), double (contours[pos])
			end else begin
				dislin, 'conmat', double (data), num_x, num_y, double (contours[pos])
			end
		end
	end
end

